import time
import threading

from mysqlog.parser import SlowQueryLog


class SlowQueryLogProducer(threading.Thread):
    def __init__(self, file, queue, **config):
        threading.Thread.__init__(self, name=file.name)
        self.parser = SlowQueryLog(file)
        self.queue = queue
        self.config = config

    def run(self):
        #
        while True:
            try:
                entry = self.parser.next()
                #
                if self.config["queryTime"] > entry["query_time"]:
                    continue
                #
                self.queue.put(entry)
            except StopIteration:
                print("No more log, just sleep 5 seconds")
                time.sleep(1)



#!/usr/bin/env python3

import typer
import queue

from mysqlog.producer import SlowQueryLogProducer
from mysqlog.consumer import SlowQueryLogConsumer

app = typer.Typer()


@app.command()
def main(
    logpath: str,
    env: str = typer.Option("default", "-e", "--env"),
    user: str = typer.Option(..., "-u", "--user"),
    password: str = typer.Option(..., "-p", "--password"),
    host: str = typer.Option("localhost", "-h", "--host"),
    port: int = typer.Option(3306, "-P", "--port"),
    database: str = typer.Option("test", "-d", "--database"),
    threadSize: int = typer.Option(1, "-t", "--thread-size"),
    since: str = typer.Option(None, "-s", "--since"),
    queryTime: float = typer.Option(0.3, "-T", "--query-time"),
):
    #
    f = open(logpath)
    q = queue.Queue(100)
    #
    producer = SlowQueryLogProducer(f, q, queryTime=queryTime)
    producer.start()
    #
    for i in range(threadSize):
        name = "SlowQueryLogConsumer" + str(i)
        #
        consumer = SlowQueryLogConsumer(
            q,
            name=name,
            env=env,
            host=host,
            port=port,
            user=user,
            password=password,
            database=database,
            since=since,
        )
        #
        consumer.start()


if __name__ == "__main__":
    app()



import hashlib
import pymysql
import threading

from datetime import datetime


class SlowQueryLogConsumer(threading.Thread):
    def __init__(self, queue, **config):
        threading.Thread.__init__(self, name=config["name"])
        self.queue = queue
        self.config = config

    def run(self):
        #
        self.connection = pymysql.connect(
            host=self.config["host"],
            port=self.config["port"],
            user=self.config["user"],
            password=self.config["password"],
            database=self.config["database"],
            cursorclass=pymysql.cursors.DictCursor,
        )
        #
        while True:
            entry = self.queue.get()
            #
            if self.config["since"] and entry["datetime"] < datetime.strptime(
                self.config["since"], "%Y-%m-%d %H:%M:%S"
            ):
                print("skip: " + str(entry))
                continue
            #
            with self.connection.cursor() as cursor:
                entry["md5"] = hashlib.md5(
                    f"{entry['datetime']}{entry['query']}{entry['query_time']}".encode("utf-8")
                ).hexdigest()
                #
                print(entry)
                #
                insert_sql = "INSERT IGNORE INTO `mysql_slow_query_log` (`md5`, `env`, `datetime`, `database`, `user`, `host`, `query`, `query_time`, `lock_time`, `rows_examined`, `rows_sent`) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
                #
                cursor.execute(
                    insert_sql,
                    (
                        entry["md5"],
                        self.config["env"],
                        entry["datetime"],
                        entry["database"],
                        entry["user"],
                        entry["host"],
                        entry["query"],
                        entry["query_time"],
                        entry["lock_time"],
                        entry["rows_examined"],
                        entry["rows_sent"],
                    ),
                )
                #
                self.connection.commit()
